<?php

/* --------------------------------------------------------------
   OrderListGeneratorInterface.inc.php 2015-11-05
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2015 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Interface OrderListGeneratorInterface
 *
 * @category   System
 * @package    Order
 * @subpackage Interfaces
 */
interface OrderListGeneratorInterface
{
	/**
	 * Returns an order list item collection.
	 *
	 * @param array      $conditions  Associative array which holds the CI_DB_query_builder conditions (default empty
	 *                                array).
	 * @param IntType    $startIndex  The start index of the wanted array to be returned (default = null).
	 * @param IntType    $maxCount    Maximum amount of items which should be returned (default = null).
	 * @param StringType $orderBy     A string which defines how the items should be ordered (default = null).
	 *
	 * @return OrderListItemCollection Order list item collection.
	 */
	public function getOrderListByConditions(array $conditions = array(),
	                                         IntType $startIndex = null,
	                                         IntType $maxCount = null,
	                                         StringType $orderBy = null);
	
	
	/**
	 * Filters records by a single keyword string.
	 *
	 * @param StringType      $keyword    Keyword string to be used for searching in order records.
	 * @param IntType|null    $startIndex The start index of the wanted array to be returned (default = null).
	 * @param IntType|null    $maxCount   Maximum amount of items which should be returned (default = null).
	 * @param StringType|null $orderBy    A string which defines how the items should be ordered (default = null).
	 *
	 * @return OrderListItemCollection Order list item collection.
	 */
	public function getOrderListByKeyword(StringType $keyword,
	                                      IntType $startIndex = null,
	                                      IntType $maxCount = null,
	                                      StringType $orderBy = null);
	
	
}
